# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from silva.fanstatic.extending import need, get_inclusion
from silva.fanstatic.resources import ExternalResource, Snippet
from fanstatic import Group
from fanstatic import get_needed

__all__ = [
    'need', 'get_inclusion', 'get_needed',
    'ExternalResource', 'Snippet', 'Group']


# Patch fanstatic with missing API calls during testing
from fanstatic.core import DummyNeededResources

DummyNeededResources.has_base_url = lambda self: True
DummyNeededResources.set_base_url = lambda self, url: None
DummyNeededResources.library_url = lambda self, library: u'/'.join(('http://test', library.name))



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import os

from martian.error import GrokError
from five import grok
from zope import component
from zope.component import provideSubscriptionAdapter
from zope.component.interface import provideInterface
from zope.interface import Interface
from zope.interface.interfaces import IInterface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IBrowserSkinType
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.interface.interface import InterfaceClass
import martian
import fanstatic
import grokcore.view
import pkg_resources

from silva.core.conf.martiansupport import directives as silvaconf
from silva.fanstatic.interfaces import ISubscribedResource
from silva.fanstatic.support import ZopeFanstaticResource
from silva.fanstatic.extending import INTERFACES_RESOURCES

_marker = object()


def is_fanstatic_resource(resource):
    """Return true if resource is a fanstatic resource.
    """
    return (isinstance(resource, fanstatic.Resource) or
            isinstance(resource, fanstatic.GroupResource))


def get_fanstatic_resource(library, resources, dependencies=[]):
    """Return a fanstatic resource from library out of the resources
    list.
    """
    dependencies = list(dependencies)
    for resource in resources:
        if is_fanstatic_resource(resource):
            dependencies.append(resource)
        elif resource in library.known_resources:
            dependencies.append(library.known_resources[resource])
        else:
            filename, extension = os.path.splitext(resource)
            minified = '.min'.join((filename, extension),)
            if not os.path.isfile(os.path.join(library.path, minified)):
                minified = None
            dependencies.append(fanstatic.Resource(
                    library, resource, minified=minified, depends=dependencies))
    if dependencies:
        return fanstatic.Group(dependencies)
    return None


def get_package_version(module_info):
    """Return the package version by looking for an egg in
    pkg_resources where the code is, returning the version of the egg.
    """
    path = module_info.path
    for egg in pkg_resources.working_set:
        if (path.startswith(egg.location) and
            path[len(egg.location)] == os.path.sep):
            if 'dev' not in egg.version:
                # We ignore this for development eggs.
                return egg.version
    return None

def get_fanstatic_library(module_info, name=None, path=None):
    """Return the fanstatic library associated to the given module.
    """
    if name is None:
        name = module_info.package_dotted_name
    if path is None:
        path = 'static'
    registry = fanstatic.get_library_registry()
    if name in registry:
        library = registry[name]
    else:
        library = fanstatic.Library(
            name, 'static',
            version=get_package_version(module_info))
        # Fix the correct path
        library.path = module_info.getResourcePath(path)
        # Register the new library to fanstatic
        registry.add(library)
    return library


def create_resource_subscriber(resource):
    """Return a subscription that require the resource content.
    """

    def subscriber(*args):
        resource.need()

    return subscriber


def list_base_layers(layer):
    """List all layers used by the given layer.
    """
    if IInterface.providedBy(layer) and layer.extends(IBrowserRequest):
        for base in layer.__bases__:
            if base in (
                IDefaultBrowserLayer, IBrowserSkinType, IBrowserRequest, Interface):
                continue
            need_parent = yield base
            if need_parent:
                need_base_parents = None
                base_parents = list_base_layers(base)
                while True:
                    try:
                        need_base_parents = yield base_parents.send(need_base_parents)
                    except StopIteration:
                        break


class ResourceIncludeGrokker(martian.InstanceGrokker):
    martian.component(InterfaceClass)

    # Class dict
    resources = {}

    def grok(self, name, interface, module_info, config, **kw):
        resources = silvaconf.resource.bind(default=_marker).get(interface)
        if resources is _marker:
            return False
        if not interface.extends(IDefaultBrowserLayer):
            raise GrokError(
                "A resource can be included only on a layer.", interface)

        # Get the fanstatic library
        library = get_fanstatic_library(module_info)

        # Create a group with all the resources
        self.resources[interface.__identifier__] = (interface, library, resources)

        config.action(
            discriminator = ('solve_dependencies', interface),
            callable = self.solve_dependencies,
            args = (interface, library, resources))
        return True

    def solve_dependencies(self, interface, library, resources):
        """This solve dependencies for resources mapped to an
        interface in the given library. When solved, a fanstatic
        resource is created and registered.
        """
        dependencies = []
        try:
            need_parent = None
            parents = list_base_layers(interface)
            while True:
                dependency = parents.send(need_parent)
                identifier = dependency.__identifier__
                if identifier in self.resources:
                    if identifier in INTERFACES_RESOURCES:
                        fanstatic_dependency = INTERFACES_RESOURCES[identifier]
                    else:
                        fanstatic_dependency = self.solve_dependencies(*self.resources[identifier])
                    if fanstatic_dependency is not None:
                        dependencies.insert(0, fanstatic_dependency)
                    need_parent = False
                else:
                    need_parent = True
        except StopIteration:
            fanstatic_resource = get_fanstatic_resource(library, resources, dependencies)
            context = silvaconf.only_for.bind().get(interface)
            factory = create_resource_subscriber(fanstatic_resource)

            provideSubscriptionAdapter(factory, (interface, context), ISubscribedResource)
            provideInterface('', interface)
            INTERFACES_RESOURCES[interface.__identifier__] = fanstatic_resource
            return fanstatic_resource
        return None



def create_factory(library):
    """Return a resource factory to browser the associated library.
    """

    def factory(request):
        return ZopeFanstaticResource(request, library)

    return factory


class StaticDirectoryGrokker(martian.GlobalGrokker):
    """Support for default resource static directory into a package.
    """

    def grok(self, name, module, module_info, config, **kw):
        # we're only interested in static resources if this module
        # happens to be a package
        if not module_info.isPackage():
            return False

        # Get the fanstatic library
        library = get_fanstatic_library(module_info)

        factory = create_factory(library)
        adapts = (IBrowserRequest,)
        provides = Interface
        config.action(
            discriminator = ('adapter', adapts, provides, library.name),
            callable = component.provideAdapter,
            args = (factory, adapts, provides, library.name))
        return True


class DirectoryResourceGrokker(martian.ClassGrokker):
    """Support for user-hand defined resource directory.
    """
    martian.component(grok.DirectoryResource)
    martian.directive(grokcore.view.name, default=None)
    martian.directive(grokcore.view.path)

    def grok(self, name, factory, module_info, **kw):
        # Need to store the module info object on the directory resource
        # class so that it can look up the actual directory.
        factory.module_info = module_info
        return super(DirectoryResourceGrokker, self).grok(
            name, factory, module_info, **kw)

    def execute(self, factory, config, name, path, **kw):
        # Get the fanstatic library
        library = get_fanstatic_library(factory.module_info, name, path)

        factory = create_factory(library)
        adapts = (IBrowserRequest,)
        provides = Interface
        config.action(
            discriminator = ('adapter', adapts, provides, library.name),
            callable = component.provideAdapter,
            args = (factory, adapts, provides, library.name))
        return True




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from zope.interface import Interface


class ISubscribedResource(Interface):
    pass


class IZopeResource(Interface):

    def get(name, default):
        pass

    def __getitem__(self, name):
        pass




from fanstatic.core import inclusion_renderers, get_needed, normalize_groups
from fanstatic.core import UnknownResourceExtensionError
from fanstatic.core import Renderable


class DummyLibrary(object):

    def __init__(self, library_nr):
        self.name = 'dummy'
        self.library_nr = library_nr

    def init_library_nr(self):
        pass

    def signature(self, **options):
        return u''


class ExternalResource(Renderable):

    def __init__(self, url, category='js', depends=None, bottom=False):
        """External resource. category can be js, css or ico.
        """
        dependency_nr = 0
        self.url = url
        self.ext = '.' + category
        self.depends = set()
        self.resources = set([self])
        if depends is not None:
            self.depends.update(normalize_groups(depends))
            for depend in self.depends:
                self.resources.update(depend.resources)
                dependency_nr = max(depend.dependency_nr + 1,
                                    dependency_nr)
        self.supersedes = []
        self.rollups = []
        self.relpath = None     # Used by sort_components
        self.library = DummyLibrary(dependency_nr)
        self.dependency_nr = dependency_nr
        self.bottom = bottom
        self.dont_bundle = True # We can never bundle this
        if self.ext not in inclusion_renderers:
            raise UnknownResourceExtensionError(
                "Unknown resource extension %s for url %s" % (
                    self.ext, url))
        self.order, self.renderer = inclusion_renderers[self.ext]

    def __eq__(self, other):
        if isinstance(other, ExternalResource):
            return other.url == self.url
        return False

    def __hash__(self):
        # Include resource url inside the hash
        return hash(('ExternalResource', self.url))

    def __eq__(self, other):
        if isinstance(other, ExternalResource):
            return other.url == self.url
        return False

    def mode(self, mode):
        return self

    def render(self, _):
        return self.renderer(self.url)

    def need(self):
        needed = get_needed()
        needed.need(self, None)

    def __repr__(self):
        return "<ExternalResource '%s'>" % (self.url)


class Snippet(Renderable):
    TEMPLATES = {
        '.js': '<script type="text/javascript">%s</script>',
        '.css': '<style type="text/css">%s</style>',
        }

    def __init__(self, snippet, category='js', depends=None, bottom=False):
        """A snippet. Category can be js or css.
        """
        dependency_nr = 0
        self.snippet = snippet
        self.ext = '.' + category
        self.depends = set()
        self.resources = set([self])
        if depends is not None:
            self.depends.update(normalize_groups(depends))
            for depend in self.depends:
                self.resources.update(depend.resources)
                dependency_nr = max(depend.dependency_nr + 1,
                                    dependency_nr)
        self.supersedes = []
        self.rollups = []
        self.relpath = None     # Used by sort_components
        self.library = DummyLibrary(dependency_nr)
        self.dependency_nr = dependency_nr
        self.bottom = bottom
        self.dont_bundle = True # We can never bundle this
        if self.ext not in inclusion_renderers:
            raise UnknownResourceExtensionError(
                "Unknown resource type %s" % self.ext)
        self.order, _ = inclusion_renderers[self.ext]

    def __hash__(self):
        # Include resource url inside the hash
        return hash(('Snippet', self.snippet))

    def __eq__(self, other):
        if isinstance(other, Snippet):
            return other.snippet == self.snippet
        return False

    def mode(self, mode):
        return self

    def render(self, _):
        template = self.TEMPLATES.get(self.ext)
        if template is not None:
            return template % self.snippet
        return ''

    def need(self):
        needed = get_needed()
        needed.need(self, None)

    def __repr__(self):
        return "<Snippet for %s>" % (self.ext[1:])




from fanstatic import compat
from fanstatic.injector import InjectorPlugin
from silva.fanstatic.extending import _set_settings
import re


class Test(object):
    pass



class URLTest(Test):
    name = 'the url ends with (path)'
    expr = re.compile(r'the url ends with (?P<path>.*)')

    def __init__(self, path):
        self.path = path

    def context(self):
        return {}

    def test(self, request, response, html):
        return request.path.endswith(self.path)


class ContainsTest(Test):
    name = 'there is (pattern)'
    expr = re.compile('there is (?P<value>.*)')

    def __init__(self, value):
        self.value = value

    def context(self):
        return {'context': self.value}

    def test(self, request, response, html):
        return self.value in html


TESTS = [URLTest, ContainsTest]


class Action(object):
    pass


class ReplaceAction(Action):
    name = 'replace (tag) with'
    expr = re.compile(r'replace ((?P<tag>.*) )?with')

    def __init__(self, tag=None):
        self.tag = tag

    def inject(self, html, resources, context):
        if self.tag is None:
            tag = context.get('context')
        else:
            tag = self.tag
        return html.replace(
            compat.as_bytestring(tag),
            compat.as_bytestring(resources))


class InjectBeforeAction(Action):
    name = 'insert before (tag)'
    expr = re.compile(r'insert before( (?P<tag>.*))?')

    def __init__(self, tag=None):
        self.tag = tag

    def inject(self, html, resources, context):
        if self.tag is None:
            tag = context.get('context')
        else:
            tag = self.tag
        return html.replace(
            compat.as_bytestring(tag),
            compat.as_bytestring(resources + '\n'+ tag))


class InjectAfterAction(Action):
    name = 'insert after (tag)'
    expr = re.compile(r'insert after( (?P<tag>.*))?')

    def __init__(self, tag=None):
        self.tag = tag

    def inject(self, html, resources, context):
        if self.tag is None:
            tag = context.get('context')
        else:
            tag = self.tag
        return html.replace(
            compat.as_bytestring(tag),
            compat.as_bytestring(tag + '\n'+ resources))

ACTIONS = [ReplaceAction, InjectBeforeAction, InjectAfterAction]


class ResourceSelector(object):
    pass


class BottomSelector(ResourceSelector):
    selector = 'bottom'

    def test(self, resource):
        return resource.bottom is True


class TopSelector(ResourceSelector):
    selector = 'top'

    def test(self, resource):
        return resource.bottom is False


class JSSelector(ResourceSelector):
    selector = 'js'

    def test(self, resource):
        return resource.ext == '.js'


class CSSSelector(ResourceSelector):
    selector = 'css'

    def test(self, resource):
        return resource.ext == '.css'


class ICOSelector(ResourceSelector):
    selector = 'ico'

    def test(self, resource):
        return resource.ext == '.ico'


SELECTORS = [BottomSelector,TopSelector,
             JSSelector, CSSSelector, ICOSelector]


class DefaultRule(object):
    default = True

    def __init__(self, action):
        self._action = action

    def context(self):
        return {}

    def inject(self, html, resources):
        return self._action.inject(html, resources, self.context())


class Rule(DefaultRule):
    default = False

    def __init__(self, action, selectors, tests):
        super(Rule, self).__init__(action)
        self._tests = tests
        self._selectors = selectors

    def context(self):
        context = {}
        for test in self._tests:
            context.update(test.context())
        return context

    def should_use(self, request, response, html):
        for test in self._tests:
            if not test.test(request, response, html):
                return False
        return True

    def do_resource_match(self, resource):
        for test in self._selectors:
            if not test.test(resource):
                return False
        return True


class ParseError(ValueError):

    def __init__(self, msg, lineno, detail=None):
        self.msg = msg
        self.lineno = lineno
        self.detail = detail

    def doc(self):
        msg = '\n\nError while reading rules of the resource injector:\n' + \
            '%s at line %d of the configuration.' % (self.msg, self.lineno)
        if self.detail:
            msg += '\n(%s.)' % (self.detail)
        return msg

    __str__ = doc


class RuleParser(object):

    def __init__(self):
        self._action = None
        self._tests = []
        self._selectors = []
        self._default = None

    def set_default(self, lineno, line):
        if self._default is not None:
            raise ParseError(
                'Duplicate default rule',
                lineno)
        if len(self._tests):
            raise ParseError(
                'Default rule specified in addition of tests',
                lineno)
        self._default = True

    def set_test(self, lineno, line):
        if self._default is not None:
            raise ParseError(
                'Test specified in addition of default rule',
                lineno)
        test = line.lstrip('if').strip()
        found_tests = []
        for provider in TESTS:
            match = provider.expr.match(test)
            if match:
                found_tests.append(provider(**match.groupdict()))
        if not len(found_tests):
            candidates = ', '.join(map(lambda a: '"%s"' % a.name, TESTS))
            raise ParseError(
                'Could not understand test "%s"'%  test,
                lineno,
                'Available tests are %s' % candidates)
        if len(found_tests) > 1:
            candidates = ', '.join(map(lambda a: a.name, found_tests))
            raise ParseError(
                'Ambiguous test "%s", ' % test,
                lineno,
                'the candidates are %s' % candidates)
        self._tests.append(found_tests[0])

    def set_action(self, lineno, action):
        if self._action is not None:
            raise ParseError(
                'Duplicate action',
                lineno)
        if 'all' in action:
            action, selectors = action.rsplit('all', 1)
            action = action.strip()
            for selector in selectors.rstrip('resources').split(' ')[1:]:
                selector = selector.strip()
                if selector:
                    for provider in SELECTORS:
                        if provider.selector == selector:
                            self._selectors.append(provider())
                            break
                    else:
                        raise ParseError(
                            'Unkown resource selector "%s"' % selector,
                            lineno)
        found_actions = []
        for provider in ACTIONS:
            match = provider.expr.match(action)
            if match:
                found_actions.append(provider(**match.groupdict()))
        if not len(found_actions):
            candidates = ', '.join(map(lambda a: '"%s"' % a.name, ACTIONS))
            raise ParseError(
                'Could not understand action "%s"' % action,
                lineno,
                'Available actions are %s' % candidates)
        if len(found_actions) > 1:
            candidates = ', '.join(map(lambda a: a.name, found_actions))
            raise ParseError(
                'Ambiguous action "%s", ' % action,
                lineno,
                'The candidates are %s' % candidates)
        self._action = found_actions[0]

    def emit(self, lineno):
        if self._action is None:
            raise ParseError('Missing action for rule', lineno)
        if self._default is True:
            if len(self._selectors):
                raise ParseError(
                    'Resource selector used in addition of the default rule',
                    lineno)
            return DefaultRule(self._action)
        return Rule(self._action, self._selectors, self._tests)


def parse_rules(config):
    if config is None:
        raise ParseError('Missing rules configuration', 0)
    current = None
    rules = []
    default = None
    for lineno, line in enumerate(config.splitlines()):
        line = line.strip()
        if not line:
            continue
        if line.startswith('if'):
            if current is None:
                current = RuleParser()
            current.set_test(lineno, line)
        elif line == 'otherwise':
            if current is None:
                current = RuleParser()
            current.set_default(lineno, line)
        else:
            if default is not None:
                raise ParseError(
                    'Rule defined after default rule', lineno)
            if current is None:
                raise ParseError(
                    'Could not understand line "%s" (missing test ?)' % line,
                    lineno)
            current.set_action(lineno, line)
            result = current.emit(lineno)
            if result.default:
                default = result
            else:
                rules.append(result)
            current = None
    if current is not None:
        raise ParseError(
            'Unfinished rule on the last line', lineno)
    return default, rules


class RuleBasedInjector(InjectorPlugin):
    name = 'rules'

    def __init__(self, options):
        _set_settings(options)
        super(RuleBasedInjector, self).__init__(options)
        self.default, self.rules = parse_rules(
            options.pop('rules',
            """otherwise
insert before </head>
"""))

    def __call__(self, html, needed, request, response=None):
        rules = []
        for rule in self.rules:
            if rule.should_use(request, response, html):
                rules.append(rule)
        includes = {}
        for resource in needed.resources():
            for rule in rules:
                if rule.do_resource_match(resource):
                    includes.setdefault(rule.inject, []).append(resource)
                    break
            else:
                if self.default is not None:
                    includes.setdefault(self.default.inject, []).append(resource)
        for injector, resources in includes.items():
            inclusion = self.make_inclusion(needed, resources)
            html = injector(html, inclusion.render())
        return html




from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from silva.core import conf as silvaconf
from silva.fanstatic import need
from five import grok


class IBaseLayer(IDefaultBrowserLayer):
    silvaconf.resource('base.css')


class IFeatureALayer(IBaseLayer):
    silvaconf.resource('featurea.css')


class IFeatureBLayer(IDefaultBrowserLayer):
    silvaconf.resource('featureb.css')


class IFullLayer(IFeatureALayer, IFeatureBLayer):
    silvaconf.resource('full.css')


class FullView(grok.View):
    grok.name('full.html')
    grok.context(Interface)

    def render(self):
        need(IFullLayer)
        return u'<html><head></head><body>Hello</body></html>'



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import logging
import Globals
import fanstatic

from zope.interface.interfaces import IInterface
from zope.testing.cleanup import addCleanUp


logger = logging.getLogger('silva.fanstatic')
INTERFACES_RESOURCES = {}
FANSTATIC_SETTINGS = {}

addCleanUp(INTERFACES_RESOURCES.clear)
addCleanUp(FANSTATIC_SETTINGS.clear)

def _set_settings(settings):
    FANSTATIC_SETTINGS.update(settings)
    mode = None
    if 'debug' in FANSTATIC_SETTINGS:
        mode = fanstatic.DEBUG
        del FANSTATIC_SETTINGS['debug']
    if 'minified' in FANSTATIC_SETTINGS:
        if mode is None:
            mode = fanstatic.MINIFIED
        del FANSTATIC_SETTINGS['minified']
    FANSTATIC_SETTINGS['mode'] = mode

def _get_settings():
    if FANSTATIC_SETTINGS:
        return FANSTATIC_SETTINGS.copy()
    if Globals.DevelopmentMode:
        return {'mode': fanstatic.DEBUG}
    return {'mode': fanstatic.MINIFIED,
            'bundle': True}

def get_inclusion(needed, resources=None):
    """Get a fanstatic ``Inclusion``. This can be used to get
    resources URL.
    """
    settings = _get_settings()
    return fanstatic.Inclusion(
        needed,
        resources=resources,
        mode=settings.get('mode', None),
        bundle=settings.get('bundle', False))


def need(resource):
    """Require given resource.
    """
    if IInterface.providedBy(resource):
        identifier = resource.__identifier__
        resource = INTERFACES_RESOURCES.get(identifier)
        if resource is None:
            logger.warning("Not including non-grokked resource %s" % identifier)
            return
        return resource.need()
    if hasattr(resource, 'need'):
        # There are no check possible ...
        resource.need()




import unittest
import doctest
from Products.Silva.testing import FunctionalLayer, suite_from_package

globs = {
    'get_browser': FunctionalLayer.get_browser,
    'grok': FunctionalLayer.grok,}


def create_test(build_test_suite, name):
    test =  build_test_suite(
        name,
        globs=globs,
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    test.layer = FunctionalLayer
    return test


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(
        suite_from_package('silva.fanstatic.tests.dependencies', create_test))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from __future__ import absolute_import

from five import grok
from zope.interface import Interface, implements
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.traversing.interfaces import ITraversable
import fanstatic

from silva.fanstatic.interfaces import ISubscribedResource
from silva.fanstatic.interfaces import IZopeResource
from silva.core.views.interfaces import IVirtualSite

from ZPublisher.interfaces import IPubFailure, IPubSuccess

_marker = object()

class ZopeFanstaticResource(object):
    # Hack to get ++resource++foo/bar/baz.jpg *paths* working in Zope
    # Pagetemplates. Note that ++resource+foo/bar/baz.jpg *URLs* will
    # not work with this hack!
    #
    # The ZopeFanstaticResource class also implements an __getitem__()
    # / get() interface, to support rendering URLs to resources from
    # code.

    implements(IZopeResource, ITraversable, IAbsoluteURL)

    def __init__(self, request, library, name=''):
        self.request = request
        self.library = library
        self.name = name

    def get(self, name, default=_marker):
        # XXX return default if given, or NotFound (or something) when
        # not, in case name is not resolved to an actual resource.
        name = '%s/%s' % (self.name, name)
        return ZopeFanstaticResource(self.request, self.library, name=name)

    def traverse(self, name, furtherPath):
        return self.get(name)

    def __getitem__(self, name):
        resource = self.get(name, None)
        if resource is None:
            raise KeyError(name)
        return resource

    def __str__(self):
        needed = fanstatic.get_needed()
        if not needed.has_base_url():
            needed.set_base_url(IVirtualSite(self.request).get_root_url())
        return needed.library_url(self.library) + self.name

    __call__ = __str__


class Resources(grok.ViewletManager):
    grok.context(Interface)
    grok.name('resources')

    def update(self):
        pass

    def render(self):
        grok.queryMultiSubscriptions(
            (self.request, self.context), ISubscribedResource)
        return u''


@grok.subscribe(IPubSuccess)
@grok.subscribe(IPubFailure)
def set_base_url(event):
    needed = fanstatic.get_needed()
    if not needed.has_resources():
        return
    if not needed.has_base_url():
        needed.set_base_url(IVirtualSite(event.request).get_root_url())



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from setuptools import setup, find_packages
import os

version = '1.2'

tests_require = [
    'infrae.wsgi [test]',
    'infrae.testbrowser',
    ]

setup(name='silva.fanstatic',
      version=version,
      description="Integration of fanstatic in Silva CMS",
      long_description=open("README.txt").read() + "\n" +
      open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Zope2",
        ],
      keywords='zope2 fanstatic silva',
      author='Sylvain Viollon',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.fanstatic',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['silva'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'fanstatic > 0.11',
        'five.grok',
        'martian',
        'silva.core.conf',
        'silva.core.views',
        'zope.component',
        'zope.interface',
        'zope.publisher',
        'zope.traversing',
        'zope.testing',
        ],
      entry_points={
        'fanstatic.injectors': [
            'rules = silva.fanstatic.injector:RuleBasedInjector',
        ],
        },
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )




from five import grok
from zope.interface import Interface

from silva.fanstatic.resources import ExternalResource


class FullView(grok.View):
    grok.name('full.html')
    grok.context(Interface)

    def render(self):
        framework = ExternalResource(
            'http://silvacms.org/framework.js')
        js = ExternalResource(
            'http://silvacms.org/tracker.js',
            depends=[framework])
        js.need()
        js.need()               # We add it twice
        css = ExternalResource(
            'http://infr.ae/tracker.css',
            category='css')
        css.need()
        return u'<html><head></head><body>Hello</body></html>'




import unittest
from silva.fanstatic import Snippet, ExternalResource

CSS = "body {color: red}"
JS_URL = "http://silvacms.org/tracker.js"


class ResourceTestCase(unittest.TestCase):

    def test_snippet(self):
        # If two Snippet are added inside the same set, you
        # only get 1.
        css = Snippet(CSS, category='css')
        self.assertEqual(
            css.ext,
            '.css')
        self.assertEqual(
            len(set([Snippet(CSS, category='css'), css])),
            1)
        self.assertEqual(
            css.render(None),
            '<style type="text/css">body {color: red}</style>')

    def test_external_resource(self):
        # If two ExternalResources are added inside the same set, you
        # only get 1.
        js = ExternalResource(JS_URL, category='js')
        self.assertEqual(
            js.ext,
            '.js')
        self.assertEqual(
            len(set([ExternalResource(JS_URL, category='js'), js])),
            1)
        self.assertEqual(
            js.render(None),
            '<script type="text/javascript" src="http://silvacms.org/tracker.js"></script>')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ResourceTestCase))
    return suite
